package com.intland.codebeamer.remoting.sample;

import java.io.*;
import java.net.MalformedURLException;

import com.intland.codebeamer.persistence.dto.*;
import com.intland.codebeamer.persistence.util.BinaryStreamDtoHelper;
import com.intland.codebeamer.remoting.*;
import com.intland.codebeamer.remoting.bean.*;


/**
 * This program is part of the CodeBeamer SDK.
 * <p>
 * This console application illustrates:
 * <ul>
 *   <li>connecting to CodeBeamer</li>
 *   <li>signing in as a CodeBeamer user</li>
 *   <li>uploading image attachments to an existing Wiki page</li>
 * </ul>
 *
 * @author <a href="mailto:robert.enyedi@intland.com">Robert Enyedi</a>
 * @version $Id$
 */
public class WikiAttachmentUploader {
	/** Main entry point. */
	public static void main(String[] args) throws Exception {
		if(args.length != 5) {
			System.err.println("Usage: <program> service-URL login password page-id image-dir");
			System.err.println("From the image directory all the GIF, JPG and PNG images " +
								"will be uploaded and attached to the specified page.");
			System.exit(-1);
		}
		String serviceUrl = args[0];
		String login = args[1];
		String password = args[2];
		String pageId = args[3];
		String imageDir = args[4];

		upload(serviceUrl, login, password, pageId, imageDir);
	}

	private static void upload(String serviceUrl, String login, String password, String pageId, String imageDir) throws MalformedURLException {
		System.out.println("Connecting to CodeBeamer web service at " + serviceUrl + "...");
		RemoteApi api = RemoteApiFactory.getInstance().connect(serviceUrl);
		if(api == null) {
			System.err.println("Couldn't connect, is the service URL correct?");
			System.exit(-1);
		}

		System.out.println("Signing in...");
		String token = api.login(login, password);
		ServerInfo serverInfo = api.getServerInfo();
		System.out.println("Signed in to CodeBeamer " + serverInfo.getMajorVersion() + serverInfo.getMinorVersion() + " (" + serverInfo.getBuildDate() + ") running on " + serverInfo.getOs() + "/Java " + serverInfo.getJavaVersion());
		UserDto user = api.getSessionUser(token);

			WikiPageDto wikiPage = api.findWikiPageById(token, new Integer(Integer.parseInt(pageId)));

			if(wikiPage == null) {
				System.out.println("Error: Could not find wiki page with name " + pageId);
			} else {
				System.out.println("Wiki page found. Creating attachments.");

				File file = new File(imageDir);

				if(file.exists() && file.isDirectory()) {
					File[] imageFiles = file.listFiles(new FilenameFilter() {

						public boolean accept(File dir, String name) {
							String lowerCaseName = name.toLowerCase();

							return	lowerCaseName.endsWith(".jpg")
									|| lowerCaseName.endsWith(".gif")
									|| lowerCaseName.endsWith(".png");
						}
					});

					System.out.println("Starting upload for " + imageFiles.length + " image files.");

					for(int i=0;i<imageFiles.length;i++) {

						System.out.println("Uploading file " + (i+1) + "/"
											+ imageFiles.length + ": "
											+ imageFiles[i].getName()
											+ " (" + (imageFiles[i].length()/1024) + " KB)");

						WikiPageCommentDto comment;
						try {
							File file2 = imageFiles[i];
							comment = createWikiPageComment(	wikiPage.getProject(),
													user,
													wikiPage,
													file2);

							comment = api.createWikiPageComment(token, comment);
							// save the blob data too
							BinaryStreamDto blob = BinaryStreamDtoHelper.loadFromFile(file2);
							api.saveArtifactCommentBlob(token, comment.getId(), blob);

							if(comment == null) {
								System.out.println("Comment creation failed.");
							}
						} catch (FileNotFoundException e) {
							e.printStackTrace();
						} catch (IOException e) {
							e.printStackTrace();
						}
					}
				} else {
					System.out.println("Error: Could not find image directory.");
				}
			}

		System.out.println("Signing out...");
		api.logout(token);

		System.out.println("Done");
	}

	public static WikiPageCommentDto createWikiPageComment(ProjectDto project, UserDto user, WikiPageDto wikiPage, File file) {
		WikiPageCommentDto comment = new WikiPageCommentDto();
		comment.setArtifact(createArtifact(project, user));
		comment.setDescription("Image attachment uploaded using the RemoteAPI.");

		comment.setFilename(file.getName());
		comment.setMimeType(getMimeType(file.getName()));
		comment.setSubmitter(user);
		comment.setWikiPage(wikiPage);

		return comment;
	}

	private static String getMimeType(String name) {
		String lowercaseName = name.toLowerCase();

		if(lowercaseName.endsWith(".jpg")) {
			return "image/jpeg";
		}

		return "image/" + lowercaseName.substring(lowercaseName.lastIndexOf('.') + 1);
	}

	public static ArtifactDto createArtifact(ProjectDto project, UserDto user) {
		ArtifactAdditionalInfoDto additionalInfo = createArtifactAdditionalInfo();
		additionalInfo.setStatus(createArtifactStatus());

		ArtifactDto artifact = new ArtifactDto();
		artifact.setProject(project);
		artifact.setName("My artifact " + 1);
		artifact.setDeleted(false);
		artifact.setOwner(user);
		artifact.setAdditionalInfo(additionalInfo);
		artifact.setTypeId(new Integer(ArtifactType.FILE));
		artifact.setGenericReferenceId(new Integer(1));

		return artifact;
	}

	public static ArtifactStatusDto createArtifactStatus() {
		ArtifactStatusDto status = new ArtifactStatusDto();
		status.setName("Status " + 1);

		return status;
	}

	public static ArtifactAdditionalInfoDto createArtifactAdditionalInfo() {
		ArtifactAdditionalInfoDto additionalInfo = new ArtifactAdditionalInfoDto();
		additionalInfo.setKeywords("Keyword1, keyword2, keyword3");
		additionalInfo.setRevision("Rev. 1");
		additionalInfo.setComment("This is my favourite artifact.");

		return additionalInfo;
	}



	/** Returns the project with the given name. */
	protected static ProjectDto findProjectByName(ProjectDto projects[], String projectName) {
		for(int i = 0; i < projects.length; i++)
			if(projects[i].getName().equals(projectName))
				return projects[i];

		return null;
	}
}
